package cyou.chenx221.controller;

import cyou.chenx221.pojo.Course;
import cyou.chenx221.pojo.Score;
import cyou.chenx221.pojo.Student;
import cyou.chenx221.pojo.Teacher;
import cyou.chenx221.service.CourseService;
import cyou.chenx221.service.ScoreService;
import cyou.chenx221.service.StudentService;
import cyou.chenx221.service.TeacherService;
import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVPrinter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@RestController
@RequestMapping("/output")
public class OutputController {

    @Autowired
    private StudentService studentService;

    @Autowired
    private CourseService courseService;

    @Autowired
    private ScoreService scoreService;

    @Autowired
    private TeacherService teacherService;

    @GetMapping("/export-csv")
    public void exportCSV(HttpServletResponse response,
                          @RequestParam("type") String type,
                          @RequestParam(value = "classes",required = false,defaultValue = "null") String classes
    ) throws IOException {
        response.setHeader("Content-Disposition", "attachment; filename=export.csv");
        response.setContentType("text/csv; charset=UTF-8"); // 设置字符编码为UTF-8
        response.setCharacterEncoding("UTF-8"); // 设置响应字符编码为UTF-8

        List<String> headers;
        List<List<String>> data;

        switch (type) {
            case "student" -> {
                headers = Arrays.asList("学生ID", "学生姓名", "性别", "出生日期", "联系方式", "班级");
                List<Student> students;
                if(classes.equals("null")){
                    students = studentService.getAllStudents(); // 根据实际情况获取学生数据
                }else {
                    students = studentService.getAllStudentsT(classes);
                }
                data = transformStudentsToData(students);
            }
            case "course" -> {
                headers = Arrays.asList("课程ID", "课程名", "课程描述");
                List<Course> courses = courseService.getAllCourses(); // 根据实际情况获取课程数据
                data = transformCoursesToData(courses);
            }
            case "score" -> {
                headers = Arrays.asList("成绩ID", "学生ID", "学生姓名", "课程ID", "课程名称", "成绩");
                List<Score> scores = scoreService.getAllScores(); // 根据实际情况获取成绩数据
                data = transformScoresToData(scores);
            }
            case "teacher" -> {
                headers = Arrays.asList("教师ID", "姓名", "性别","出生日期","授课课程ID","授课课程名","授课班级");
                List<Teacher> teachers = teacherService.getAllTeachers(); // 根据实际情况获取教师数据
                data = transformTeachersToData(teachers);
            }
            default -> throw new IllegalArgumentException("Invalid export type: " + type);
        }

        // 使用 Apache Commons CSV 构建 CSV 输出
        try (OutputStream outputStream = response.getOutputStream();
             OutputStreamWriter outputStreamWriter = new OutputStreamWriter(outputStream, StandardCharsets.UTF_8);
             CSVPrinter csvPrinter = new CSVPrinter(outputStreamWriter, CSVFormat.DEFAULT)) {
            csvPrinter.printRecord(headers);
            for (List<String> row : data) {
                csvPrinter.printRecord(row);
            }
            csvPrinter.flush();
        }
    }

    // 辅助方法：将学生对象列表转换为数据列表
    private List<List<String>> transformStudentsToData(List<Student> students) {
        List<List<String>> data = new ArrayList<>();
        for (Student student : students) {
            List<String> rowData = Arrays.asList(
                    String.valueOf(student.getId()),
                    student.getName(),
                    student.getSex(),
                    student.getBirthday().toString(),
                    String.valueOf(student.getPhone()),
                    String.valueOf(student.getClasses())
            );
            data.add(rowData);
        }
        return data;
    }

    // 辅助方法：将课程对象列表转换为数据列表
    private List<List<String>> transformCoursesToData(List<Course> courses) {
        List<List<String>> data = new ArrayList<>();
        for (Course course : courses) {
            List<String> rowData = Arrays.asList(
                    String.valueOf(course.getCourseID()),
                    course.getCourseName(),
                    course.getDescription()
            );
            data.add(rowData);
        }
        return data;
    }

    // 辅助方法：将成绩对象列表转换为数据列表
    private List<List<String>> transformScoresToData(List<Score> scores) {
        List<List<String>> data = new ArrayList<>();
        for (Score score : scores) {
            List<String> rowData = Arrays.asList(
                    String.valueOf(score.getScoreID()),
                    String.valueOf(score.getStudent().getId()),
                    score.getStudent().getName(),
                    String.valueOf(score.getCourse().getCourseID()),
                    score.getCourse().getCourseName(),
                    String.valueOf(score.getScore())
            );
            data.add(rowData);
        }
        return data;
    }

    private List<List<String>> transformTeachersToData(List<Teacher> teachers) {
        List<List<String>> data = new ArrayList<>();
        for (Teacher teacher : teachers) {
            List<String> rowData = Arrays.asList(
                    String.valueOf(teacher.getId()),
                    teacher.getName(),
                    teacher.getSex(),
                    teacher.getBirthday().toString(),
                    String.valueOf(teacher.getCourse().getCourseID()),
                    teacher.getCourse().getCourseName(),
                    teacher.getClasses()
            );
            data.add(rowData);
        }
        return data;
    }

    @GetMapping("/get-csv")
    public void getCSV(HttpServletResponse response, @RequestParam("filename") String filename) throws IOException {
        String filePath = "file" + File.separator + filename;

        File file = new File(filePath);
        if (file.exists()) {
            // 设置响应头
            response.setHeader("Content-Disposition", "attachment; filename=" + file.getName());
            response.setContentType("text/csv");

            // 读取文件内容并写入响应流
            try (InputStream inputStream = new FileInputStream(file);
                 OutputStream outputStream = response.getOutputStream()) {
                byte[] buffer = new byte[1024];
                int bytesRead;
                while ((bytesRead = inputStream.read(buffer)) != -1) {
                    outputStream.write(buffer, 0, bytesRead);
                }
            } catch (IOException e) {
                // 处理异常
            }
        } else {
            System.out.println("异常");
        }
    }
}